@@ -203,7 +203,14 @@ impl Request<'_> {
         data_len: u32,
     ) -> ssh_format::Result<&'a [u8]> {
         serializer.reset();
-        (constants::SSH_FXP_WRITE, request_id, handle, offset).serialize(&mut *serializer)?;
+        (
+            constants::SSH_FXP_WRITE,
+            request_id,
+            handle,
+            offset,
+            data_len,
+        )
+            .serialize(&mut *serializer)?;
 
         serializer
             .get_output_with_data(data_len)
@@ -239,14 +246,14 @@ bitflags! {
 pub struct OpenFile<'a> {
     filename: Cow<'a, Path>,
     flags: u32,
-    attrs: Option<FileAttrs>,
+    attrs: FileAttrs,
 }
 impl<'a> OpenFile<'a> {
     pub fn open(filename: Cow<'a, Path>, mode: FileMode) -> Self {
         Self {
             filename,
             flags: mode.bits(),
-            attrs: None,
+            attrs: Default::default(),
         }
     }
 
@@ -259,7 +266,7 @@ impl<'a> OpenFile<'a> {
         Self {
             filename,
             flags: mode.bits() | constants::SSH_FXF_CREAT | create_flags.bits(),
-            attrs: Some(attrs),
+            attrs,
         }
     }
 }
