@@ -47,7 +47,7 @@ impl ServerVersion {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub enum ResponseInner {
     Status {
         status_code: StatusCode,
@@ -64,7 +64,7 @@ pub enum ResponseInner {
     Attrs(FileAttrsBox),
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct Response {
     pub response_id: u32,
     pub response_inner: ResponseInner,
@@ -199,7 +199,7 @@ impl<'de> Deserialize<'de> for StatusCode {
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Clone, Deserialize)]
 pub struct ErrMsg(TwoStrs);
 
 impl ErrMsg {
@@ -227,6 +227,12 @@ impl fmt::Display for ErrMsg {
     }
 }
 
+impl fmt::Debug for ErrMsg {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(self, f)
+    }
+}
+
 /// Entry in ResponseInner::Name
 #[derive(Debug, Deserialize, Clone)]
 pub struct NameEntry {
